import pygame


from OpenGL.GL import *
from OpenGL.GLU import gluPerspective

from cFont import Font
from cTimer import Timer
from cLevel import Level

class Game:
	def __init__(self):
		self.levels = [Level('level1')]
		self.curLevel = 0
		self.level = self.levels[self.curLevel]

		self.fpsfont = Font('graphics/font.tga')
		
		self.fps = 0
		self.fpsacc = 0
		self.fpscount = 0

		self.usejoy = 0	#default false
		self.lastPress = [False]*6

	def draw(self):
		# setup perspective drawing
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(70, 4.0/3.0, 1, 100)
		glMatrixMode(GL_MODELVIEW)
		
		glEnable(GL_DEPTH_TEST)
		
		# PERSPECTIVE -------
		# draw the levels including in-game sprites
		glClearDepth(1.0)
		glClearColor(0,0,0,0)
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

		glLoadIdentity()
		glTranslate(0,0,-10)

		self.level.drawLevel()

		glFlush()
		# -------------------

		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		glOrtho(0,640,0,480,0,1)
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()

		glDisable(GL_DEPTH_TEST)

		# ORTHO -------------
		# here we can draw some overlay sprites such as HUD etc.
		self.level.drawOverlay()


		self.fpsacc += Timer.diff
		self.fpscount += 1
		
		if self.fpscount == 10:
			self.fps = int(10.0/self.fpsacc)
			self.fpscount = 0
			self.fpsacc = 0
		
		glColor4f(1,1,1,1)
		self.fpsfont.drawtextright(str(self.fps),630,10)


		# -------------------
	def detectJoy(self,app):
		buttons = app.joystick.check(app.settings.joy,app.settings.keys,self.usejoy)
		
		up,down,left,right,action,special = 0,1,2,3,4,5

		xspeed=yspeed= 2 * Timer.diff

		hasmorphed=False
		maywalk=True
		
		crstr=self.level.hero.stretch

		clmb = self.level.hero.climb

		keyDown = []
		btnPress = []
		for i in xrange(6):
			btnPress.append(bool(buttons[i]))
			keyDown.append( btnPress[i] and not self.lastPress[i] )
		self.lastPress = btnPress

		if self.level.editMode:
			if btnPress[special]:
				if keyDown[down]: self.level.editCursor.change(-1)
				elif keyDown[up]: self.level.editCursor.change(1)
				if keyDown[left]: self.level.editCursor.changeEditTile(-1)
				elif keyDown[right]: self.level.editCursor.changeEditTile(1)
			else:
				if keyDown[0]: self.level.editCursor.move(0,1)
				elif keyDown[1]: self.level.editCursor.move(0,-1)
				if keyDown[2]: self.level.editCursor.move(-1,0)
				elif keyDown[3]: self.level.editCursor.move(1,0)
				if keyDown[action]: self.level.editCursor.toggleSolid()
		else:
			if buttons[0] == True and clmb==0: self.level.hero.stretchy(1);hasmorphed=True		#up
			if buttons[1] == True and clmb==0: self.level.hero.stretchy(-1);hasmorphed=True		#down
			if buttons[2] == True and clmb==0: 
				if(crstr<1.2 and crstr >0.8): self.level.hero.move((-xspeed,0))		#left
				elif (crstr>=1.2):
					self.level.hero.leany(-1)
			if buttons[3] == True and clmb==0: 
				if(crstr<1.2 and crstr >0.8): self.level.hero.move((xspeed,0))		#right
				elif (crstr>=1.2):
					self.level.hero.leany(1)
			if buttons[4] == True and clmb==0: pass		#special
			if buttons[5] == True and clmb==0: pass			#action
	
		if(hasmorphed==False):self.level.hero.morphback()

	def update(self,app):
		if not self.level.timer.active: return
		self.detectJoy(app)

		self.level.update()
	
	def start(self,usejoy):
		self.level.timer.start()
		self.usejoy=usejoy

	def stop(self):
		self.level.timer.stop()

	def restart(self):
		self.level.timer.reset()
		self.level.reset()
		self.level.timer.start()
		

